@@ -3,45 +3,36 @@ pub(crate) mod bi_hash_map;
 pub mod errors;
 #[doc(hidden)]
 pub mod hyperedges;
+mod indexes;
+#[doc(hidden)]
+pub mod iterator;
 mod shared;
 mod utils;
 #[doc(hidden)]
 pub mod vertices;
 
 use bi_hash_map::BiHashMap;
+// Reexport indexes at this level.
+pub use crate::core::indexes::{HyperedgeIndex, VertexIndex};
+
 use indexmap::{IndexMap, IndexSet};
 use std::{
     fmt::{Debug, Display, Formatter, Result},
     hash::Hash,
+    ops::Deref,
 };
 
-/// Shared Trait for hyperedges and vertices.
+/// Shared Trait for the vertices.
 /// Must be implemented to use the library.
-pub trait SharedTrait: Copy + Debug + Display + Eq + Hash {}
+pub trait VertexTrait: Copy + Debug + Display + Eq + Hash {}
 
-impl<T> SharedTrait for T where T: Copy + Debug + Display + Eq + Hash {}
+impl<T> VertexTrait for T where T: Copy + Debug + Display + Eq + Hash {}
 
-/// Vertex stable index representation as usize.
-/// Uses the newtype index pattern.
-/// <https://matklad.github.io/2018/06/04/newtype-index-pattern.html>
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct VertexIndex(pub usize);
+/// Shared Trait for the hyperedges.
+/// Must be implemented to use the library.
+pub trait HyperedgeTrait: VertexTrait + Into<usize> {}
 
-impl Display for VertexIndex {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
-        write!(f, "{}", self.0)
-    }
-}
-
-/// Hyperedge stable index representation as usize.
-#[derive(Clone, Copy, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct HyperedgeIndex(pub usize);
-
-impl Display for HyperedgeIndex {
-    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
-        write!(f, "{}", self.0)
-    }
-}
+impl<T> HyperedgeTrait for T where T: VertexTrait + Into<usize> {}
 
 /// A HyperedgeKey is a representation of both the vertices and the weight
 /// of a hyperedge, used as a key in the hyperedges set.
@@ -49,7 +40,7 @@ impl Display for HyperedgeIndex {
 /// different hyperedges, the weight is also included in the key to keep
 /// it unique.
 #[derive(Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
-pub struct HyperedgeKey<HE> {
+pub(crate) struct HyperedgeKey<HE> {
     vertices: Vec<usize>,
     weight: HE,
 }
@@ -61,17 +52,25 @@ impl<HE> HyperedgeKey<HE> {
     }
 }
 
+impl<HE> Deref for HyperedgeKey<HE> {
+    type Target = HE;
+
+    fn deref(&self) -> &HE {
+        &self.weight
+    }
+}
+
 /// A directed hypergraph composed of generic vertices and hyperedges.
 pub struct Hypergraph<V, HE> {
     /// Vertices are stored as a map whose unique keys are the weights
     /// and the values are a set of the hyperedges indexes which include
     // the current vertex.
-    pub vertices: IndexMap<V, IndexSet<usize>>,
+    vertices: IndexMap<V, IndexSet<usize>>,
 
     /// Hyperedges are stored as a set whose unique keys are a combination of
     /// vertices indexes and a weight. Two or more hyperedges can contain
     /// the exact same vertices (non-simple hypergraph).
-    pub hyperedges: IndexSet<HyperedgeKey<HE>>,
+    hyperedges: IndexSet<HyperedgeKey<HE>>,
 
     // Bi-directional maps for hyperedges and vertices.
     hyperedges_mapping: BiHashMap<HyperedgeIndex>,
@@ -82,7 +81,11 @@ pub struct Hypergraph<V, HE> {
     vertices_count: usize,
 }
 
-impl<V: Eq + Hash + Debug, HE: Debug> Debug for Hypergraph<V, HE> {
+impl<V, HE> Debug for Hypergraph<V, HE>
+where
+    V: Eq + Hash + Debug,
+    HE: Debug,
+{
     fn fmt(&self, f: &mut Formatter<'_>) -> Result {
         f.debug_struct("Hypergraph")
             .field("vertices", &self.vertices)
@@ -93,8 +96,8 @@ impl<V: Eq + Hash + Debug, HE: Debug> Debug for Hypergraph<V, HE> {
 
 impl<V, HE> Default for Hypergraph<V, HE>
 where
-    V: SharedTrait,
-    HE: SharedTrait,
+    V: VertexTrait,
+    HE: HyperedgeTrait,
 {
     fn default() -> Self {
         Hypergraph::new()
@@ -104,8 +107,8 @@ where
 /// Hypergraph implementations.
 impl<V, HE> Hypergraph<V, HE>
 where
-    V: SharedTrait,
-    HE: SharedTrait,
+    V: VertexTrait,
+    HE: HyperedgeTrait,
 {
     /// Clears the hypergraph.
     pub fn clear(&mut self) {
