@@ -1,15 +1,15 @@
 #![deny(unsafe_code, nonstandard_style)]
-#![forbid(rust_2018_idioms)]
+#![forbid(rust_2021_compatibility)]
 
 mod common;
 
-use common::Vertex;
+use common::{Hyperedge, Vertex};
 use hypergraph::{errors::HypergraphError, HyperedgeIndex, Hypergraph, VertexIndex};
 
 #[test]
-fn integration() {
+fn integration_main() {
     // Create a new hypergraph.
-    let mut graph = Hypergraph::<Vertex<'_>, &str>::new();
+    let mut graph = Hypergraph::<Vertex, Hyperedge>::new();
 
     // Create some vertices.
     let andrea = Vertex::new("Andrea");
@@ -53,7 +53,15 @@ fn integration() {
     // Count the vertices.
     assert_eq!(graph.count_vertices(), 5, "should have 5 vertices");
 
-    // Add some hyperedges.
+    // Create some hyperedges.
+    let first_hyperedge = Hyperedge::new("pass the pink ball", 1);
+    let second_hyperedge = Hyperedge::new("pass the yellow ball", 1);
+    let third_hyperedge = Hyperedge::new("share the \"The Disordered Cosmos: A Journey into Dark Matter, Spacetime, and Dreams Deferred\" book", 2);
+    let fourth_hyperedge = Hyperedge::new("meditate like a Jedi", 10);
+    let fifth_hyperedge = Hyperedge::new("work out", 20);
+    let sixth_hyperedge = Hyperedge::new("nope", 0);
+
+    // Add those hyperedges to the hypergraph.
     assert_eq!(
         graph.add_hyperedge(
             vec![
@@ -62,7 +70,7 @@ fn integration() {
                 VertexIndex(1),
                 VertexIndex(3)
             ],
-            "pass the pink ball"
+            first_hyperedge
         ),
         Ok(HyperedgeIndex(0)),
         "should add a first hyperedge which contains a self-loop on the VertexIndex 1"
@@ -75,7 +83,7 @@ fn integration() {
                 VertexIndex(1),
                 VertexIndex(3)
             ],
-            "pass the yellow ball",
+            second_hyperedge,
         ),
         Ok(HyperedgeIndex(1)),
         "should add a second hyperedge which contains the same vertices as the first one"
@@ -88,36 +96,38 @@ fn integration() {
                 VertexIndex(3),
                 VertexIndex(2)
             ],
-            "share the \"The Disordered Cosmos: A Journey into Dark Matter, Spacetime, and Dreams Deferred\" book"
+            third_hyperedge,
         ),
         Ok(HyperedgeIndex(2)),
         "should add a third hyperedge which is unique"
     );
     assert_eq!(
-        graph.add_hyperedge(vec![VertexIndex(3)], "meditate like a Jedi"),
+        graph.add_hyperedge(vec![VertexIndex(3)], fourth_hyperedge,),
         Ok(HyperedgeIndex(3)),
         "should add a fourth hyperedge which contains a unary"
     );
     assert_eq!(
-        graph.add_hyperedge(vec![VertexIndex(3)], "meditate like a Jedi"),
+        graph.add_hyperedge(vec![VertexIndex(3)], fourth_hyperedge,),
         Err(HypergraphError::HyperedgeWeightAlreadyAssigned(
-            "meditate like a Jedi"
+            fourth_hyperedge
         )),
         "should return an explicit error since this weight is already in use"
     );
     assert_eq!(
-        graph.add_hyperedge(vec![VertexIndex(3)], "work out"), 
-        Ok(HyperedgeIndex(4)),
-        "should add a fifth hyperedge which contains the same unary as the fourth one but with a different weight"
-    );
+            graph.add_hyperedge(vec![VertexIndex(3)], fifth_hyperedge),
+            Ok(HyperedgeIndex(4)),
+            "should add a fifth hyperedge which contains the same unary as the fourth one but with a different weight"
+        );
     assert_eq!(
-        graph.add_hyperedge(vec![VertexIndex(9)], "nope"),
+        graph.add_hyperedge(vec![VertexIndex(9)], sixth_hyperedge),
         Err(HypergraphError::VertexIndexNotFound(VertexIndex(9))),
         "should be out-of-bound and return an explicit error"
     );
     assert_eq!(
-        graph.add_hyperedge(vec![], "nope"),
-        Err(HypergraphError::HyperedgeCreationNoVertices("nope")),
+        graph.add_hyperedge(vec![], sixth_hyperedge),
+        Err(HypergraphError::HyperedgeCreationNoVertices(
+            Hyperedge::new("nope", 0)
+        )),
         "should return an explicit error since the vertices are missing"
     );
 
@@ -127,12 +137,12 @@ fn integration() {
     // Get the weights of some vertices.
     assert_eq!(
         graph.get_vertex_weight(VertexIndex(0)),
-        Ok(andrea),
+        Ok(&andrea),
         "should return Andrea"
     );
     assert_eq!(
         graph.get_vertex_weight(VertexIndex(4)),
-        Ok(enola),
+        Ok(&enola),
         "should return Enola"
     );
     assert_eq!(
@@ -144,12 +154,12 @@ fn integration() {
     // Get the weights of some hyperedges.
     assert_eq!(
         graph.get_hyperedge_weight(HyperedgeIndex(0)),
-        Ok("pass the pink ball"),
+        Ok(&Hyperedge::new("pass the pink ball", 1)),
         "should get the weight of the first hyperedge"
     );
     assert_eq!(
         graph.get_hyperedge_weight(HyperedgeIndex(4)),
-        Ok("work out"),
+        Ok(&Hyperedge::new("work out", 20)),
         "should get the weight of the fifth hyperedge"
     );
     assert_eq!(
@@ -374,53 +384,117 @@ fn integration() {
         graph.get_adjacent_vertices_from(VertexIndex(0)),
         Ok(vec![VertexIndex(1), VertexIndex(3)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_from(VertexIndex(0)),
+        Ok(vec![
+            (VertexIndex(1), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+            (VertexIndex(3), vec![HyperedgeIndex(2)]),
+        ])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_from(VertexIndex(1)),
         Ok(vec![VertexIndex(1), VertexIndex(3)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_from(VertexIndex(1)),
+        Ok(vec![
+            (VertexIndex(1), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+            (VertexIndex(3), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+        ])
+    );
     assert_eq!(graph.get_adjacent_vertices_from(VertexIndex(2)), Ok(vec![]));
+    assert_eq!(
+        graph.get_full_adjacent_vertices_from(VertexIndex(2)),
+        Ok(vec![])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_from(VertexIndex(3)),
         Ok(vec![VertexIndex(2)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_from(VertexIndex(3)),
+        Ok(vec![(VertexIndex(2), vec![HyperedgeIndex(2)])])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_from(VertexIndex(5)),
         Err(HypergraphError::VertexIndexNotFound(VertexIndex(5))),
         "should be out-of-bound and return an explicit error"
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_from(VertexIndex(5)),
+        Err(HypergraphError::VertexIndexNotFound(VertexIndex(5))),
+        "should be out-of-bound and return an explicit error"
+    );
 
     // Get the adjacent vertices to a vertex.
     assert_eq!(
         graph.get_adjacent_vertices_to(VertexIndex(0)),
         Ok(vec![VertexIndex(4)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_to(VertexIndex(3)),
+        Ok(vec![
+            (VertexIndex(1), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+            (VertexIndex(0), vec![HyperedgeIndex(2)]),
+        ])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_to(VertexIndex(1)),
         Ok(vec![VertexIndex(0), VertexIndex(1)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_to(VertexIndex(1)),
+        Ok(vec![
+            (VertexIndex(0), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+            (VertexIndex(1), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+        ])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_to(VertexIndex(2)),
         Ok(vec![VertexIndex(3)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_to(VertexIndex(2)),
+        Ok(vec![(VertexIndex(3), vec![HyperedgeIndex(2)])])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_to(VertexIndex(3)),
         Ok(vec![VertexIndex(0), VertexIndex(1)])
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_to(VertexIndex(3)),
+        Ok(vec![
+            (VertexIndex(1), vec![HyperedgeIndex(0), HyperedgeIndex(1)]),
+            (VertexIndex(0), vec![HyperedgeIndex(2)])
+        ])
+    );
     assert_eq!(
         graph.get_adjacent_vertices_to(VertexIndex(5)),
         Err(HypergraphError::VertexIndexNotFound(VertexIndex(5))),
         "should be out-of-bound and return an explicit error"
     );
+    assert_eq!(
+        graph.get_full_adjacent_vertices_to(VertexIndex(5)),
+        Err(HypergraphError::VertexIndexNotFound(VertexIndex(5))),
+        "should be out-of-bound and return an explicit error"
+    );
 
     // Get some paths via Dijkstra.
     assert_eq!(
         graph.get_dijkstra_connections(VertexIndex(4), VertexIndex(1)),
-        Ok(vec![VertexIndex(4), VertexIndex(0), VertexIndex(1),]),
+        Ok(vec![
+            (VertexIndex(4), None),
+            (VertexIndex(0), Some(HyperedgeIndex(2))),
+            (VertexIndex(1), Some(HyperedgeIndex(0)))
+        ]),
         "should get a path of three vertices"
     );
     assert_eq!(
         graph.get_dijkstra_connections(VertexIndex(0), VertexIndex(3)),
-        Ok(vec![VertexIndex(0), VertexIndex(3)]),
+        Ok(vec![
+            (VertexIndex(0), None),
+            (VertexIndex(3), Some(HyperedgeIndex(2)))
+        ]),
         "should get a path of two vertices"
     );
     assert_eq!(
@@ -430,12 +504,12 @@ fn integration() {
     );
     assert_eq!(
         graph.get_dijkstra_connections(VertexIndex(1), VertexIndex(1)),
-        Ok(vec![VertexIndex(1)]),
+        Ok(vec![(VertexIndex(1), None)]),
         "should get a path of one vertex"
     );
     assert_eq!(
         graph.get_dijkstra_connections(VertexIndex(3), VertexIndex(3)),
-        Ok(vec![VertexIndex(3)]),
+        Ok(vec![(VertexIndex(3), None)]),
         "should get a path of one vertex"
     );
     assert_eq!(
@@ -449,7 +523,7 @@ fn integration() {
     assert_eq!(graph.update_vertex_weight(VertexIndex(1), bjǫrg), Ok(()));
     assert_eq!(
         graph.get_vertex_weight(VertexIndex(1)),
-        Ok(bjǫrg),
+        Ok(&bjǫrg),
         "should return Bjǫrg instead of Bjǫrn"
     );
     assert_eq!(graph.count_vertices(), 5, "should still have 5 vertices");
@@ -457,14 +531,15 @@ fn integration() {
     // Update a hyperedge's weight.
     // First case: the index is the last one, no internal index alteration
     // occurs.
+    let fifth_hyperedge = Hyperedge::new("sleep", 0);
     assert_eq!(
-        graph.update_hyperedge_weight(HyperedgeIndex(4), "sleep"),
+        graph.update_hyperedge_weight(HyperedgeIndex(4), fifth_hyperedge),
         Ok(()),
         "should update the weight of the fifth hyperedge"
     );
     assert_eq!(
         graph.get_hyperedge_weight(HyperedgeIndex(4)),
-        Ok("sleep"),
+        Ok(&fifth_hyperedge),
         "should get the new weight of the fifth hyperedge"
     );
     assert_eq!(
@@ -474,14 +549,15 @@ fn integration() {
     );
     // Second case: the index is not the last one, an internal index alteration
     // occurs but is anyway fixed by the insertion.
+    let first_hyperedge = Hyperedge::new("pass the purple ball", 3);
     assert_eq!(
-        graph.update_hyperedge_weight(HyperedgeIndex(0), "pass the purple ball"),
+        graph.update_hyperedge_weight(HyperedgeIndex(0), first_hyperedge),
         Ok(()),
         "should update the weight of the first hyperedge"
     );
     assert_eq!(
         graph.get_hyperedge_weight(HyperedgeIndex(0)),
-        Ok("pass the purple ball"),
+        Ok(&first_hyperedge),
         "should get the new weight of the first hyperedge"
     );
     assert_eq!(
@@ -491,17 +567,17 @@ fn integration() {
     );
     // Check the eventual errors.
     assert_eq!(
-        graph.update_hyperedge_weight(HyperedgeIndex(0), "pass the purple ball"),
+        graph.update_hyperedge_weight(HyperedgeIndex(0), first_hyperedge),
         Err(HypergraphError::HyperedgeWeightUnchanged {
             index: HyperedgeIndex(0),
-            weight: "pass the purple ball"
+            weight: first_hyperedge,
         }),
         "should return an explicit error since this weight has not changed"
     );
     assert_eq!(
-        graph.update_hyperedge_weight(HyperedgeIndex(0), "meditate like a Jedi"),
+        graph.update_hyperedge_weight(HyperedgeIndex(0), fourth_hyperedge),
         Err(HypergraphError::HyperedgeWeightAlreadyAssigned(
-            "meditate like a Jedi"
+            fourth_hyperedge
         )),
         "should return an explicit error since this weight is already assigned"
     );
